searchData={"content_type":"text/markdown","items":[{"doc":"Adds XML parsing and generation to modules via `use XmlSchema`.\n\nDefine Xml structures with:\n- [xml](#xml/1)\n- [xml_tag](#xml_tag/3)\n- [xml_one](#xml_one/3)\n- [xml_many](#xml/3)\n\nOptions to `use`:\n  `tag:  ` string to use as xml tag name for this element.\n   This allows a tag name not derived from the module name.\n\n  `print:  ` will print the macro generated code, e.g:\n    `print: Mix.env() in [:test, :dev]`\n\n\n# Generated functions\n\nAn XmlSchema will provide the generated functions:\n- parse_xml/1 - accepts an xml string to parse\n- xml_tag/0 - returns the tag used when generating xml from this node\n- xml_tag_list - order list of tags in this schema\n- transform/1 - overridable function that can be used to transform tag names during parsing\n\n# Example\n\n```elixir\ndefmodule Simple do\n  use XmlSchema, xml_name: \"a\"\n  xml do\n    xml_tag :x, :string\n    xml_tag :y, :boolean\n    xml_one :z, Z do\n      xml_tag :a, :string\n      xml_tag :b, :string\n    end\n    xml_many :j, J do\n      xml_tag :q, :string\n    end\n    xml_tag :g, {:array, :string}\n  end\nend\n```\n\nWill parse\n\n```xml\n \n \n   hill \n   false \n   \n     tree \n     bush \n   \n   \n     cat \n   \n   \n     dog \n   \n   hippo \n   elephant \n   rhino \n \n```\n\nInto:\n\n```elixir\n%Simple{\n  _attributes: %{\"otherattr\" => \"red\", \"someattr\" => \"blue\"},\n  x: \"hill\",\n  y: false,\n  z: %Simple.Z{_attributes: nil, a: \"tree\", b: \"bush\"},\n  j: [\n    %Simple.J{_attributes: nil, q: \"cat\"},\n    %Simple.J{_attributes: nil, q: \"dog\"}\n  ],\n  g: [\"hippo\", \"elephant\", \"rhino\"]\n}\n```\n\n# Attributes\n\nParsed attibutes are stored in a map, this precludes multiple same named\nattribute in a single tag. It also does not preserve attribute ordering.\n\nTerminal nodes defined with `xml_tag` provide bare values. If you wish to\ncapture attributes the type argument takes the form of `{:param, type}` which\nwill result in not bare values but a tuple of `{value, %{attribute map}}`.\n\n\nAn example:\n```elixir\ndefmodule Example.Attribute do\n  use XmlSchema, tag: \"a\"\n\n  xml do\n    xml_tag :k, :string\n    xml_tag :j, {:param, :string}\n    xml_tag :manyj, {:array, {:param, :string}}\n  end\n\n  def doc do\n    \"\"\"\n       \n         one \n         tree \n         shrub \n         flower \n       \n    \"\"\"\n  end\n\n  def expect do\n    %Example.Attribute{\n      _attributes: %{\"first\" => \"lemon\", \"second\" => \"lime\"},\n      k: \"one\",\n      j: {\"tree\", %{\"fourth\" => \"orange\"}},\n      manyj: [\n        {\"shrub\", %{\"five\" => \"cherry\"}},\n        {\"flower\", %{\"six\" => \"berry\"}}\n      ]\n    }\n  end\n\n  def expect_xml do\n    \"\"\"\n     \n     \n       one \n       tree \n       shrub \n       flower \n     \n    \"\"\"\n    |> String.trim_trailing()\n  end\nend\n\n```\n\n\nFor more examples, see test/support.","ref":"XmlSchema.html","title":"XmlSchema","type":"module"},{"doc":"Generate xml document from xml schema","ref":"XmlSchema.html#generate_xml/1","title":"XmlSchema.generate_xml/1","type":"function"},{"doc":"Generate xml document from xml schema with `name` top tag","ref":"XmlSchema.html#generate_xml/2","title":"XmlSchema.generate_xml/2","type":"function"},{"doc":"Parse an xml string based on the structure of `module`, which must be an\nXmlSchema. This function is added to any module that uses XmlSchema and\nis simplest to call the function there instead of this one.","ref":"XmlSchema.html#parse_xml/2","title":"XmlSchema.parse_xml/2","type":"function"},{"doc":"Start definition of xml document structure.","ref":"XmlSchema.html#xml/1","title":"XmlSchema.xml/1","type":"macro"},{"doc":"Similar to [`xml_one/3`](#xml_one/3) except the tag can occur 0 or\nmore times. Values will be presented in a list. Unlike `xml_one` if\nthe tag is not present an empty list will result.\n\nIn this example (from test/support/example/many.ex),\n`doc` is an xml sample that will result in the `expect` struct.\n\n```\ndefmodule Example.Many do\n  use XmlSchema, tag: \"a\"\n\n  xml do\n    xml_many :j, Example.Tag\n    xml_many :s, Example.Tag\n  end\n\n  def doc do\n    \"\"\"\n       \n         \n           one \n         \n         \n           two \n         \n       \n    \"\"\"\n  end\n\n  def expect do\n    %Example.Many{\n      _attributes: nil,\n      j: [\n        %Example.Tag{_attributes: nil, k: \"one\"},\n        %Example.Tag{_attributes: nil, k: \"two\"}\n      ],\n      s: []\n    }\n  end\n  def expect_xml do\n    \"\"\"\n     \n     \n       \n         one \n       \n       \n         two \n       \n     \n    \"\"\"\n    |> String.trim_trailing()\n  end\nend\n\n```","ref":"XmlSchema.html#xml_many/3","title":"XmlSchema.xml_many/3","type":"macro"},{"doc":"Declare a tag that occurs once with XML tag `name` as either\na block or a declared module. Modules generated by blocks automatically\n`use XmlSchema`. nil value is used if a tag is not present.\n\nBlock form example:\n\n```\ndefmodule Example.OneBlock do\n  use XmlSchema, tag: \"a\"\n\n  xml do\n    xml_one :j, Tag do\n      xml_tag :k, :string\n    end\n  end\n\n  def doc do\n    \"\"\"\n     \n       \n         one \n       \n     \n    \"\"\"\n  end\n\n  def expect do\n    %Example.OneBlock{\n      _attributes: nil,\n      j: %Example.OneBlock.Tag{_attributes: nil, k: \"one\"}\n    }\n  end\n\n  def expect_xml do\n    \"\"\"\n     \n     \n       \n         one \n       \n     \n    \"\"\"\n    |> String.trim_trailing()\n  end\nend\n\n```\n\nIn this example (from test/support/example/one.ex)\n`doc` is an xml sample that will result in the `expect` struct.\n\nDefined module form:\n```\ndefmodule Example.One do\n  use XmlSchema, tag: \"a\"\n\n  xml do\n    xml_one :j, Example.Tag\n    xml_one :s, Example.Tag\n  end\n\n  def doc do\n    \"\"\"\n     \n       \n         one \n       \n     \n    \"\"\"\n  end\n\n  def expect do\n    %Example.One{\n      _attributes: nil,\n      j: %Example.Tag{_attributes: nil, k: \"one\"},\n      s: nil\n    }\n  end\n\n  def expect_xml do\n    \"\"\"\n     \n     \n       \n         one \n       \n     \n    \"\"\"\n    |> String.trim_trailing()\n  end\nend\n\n```\n\nDepends on the definition of module `Example.Tag`.","ref":"XmlSchema.html#xml_one/3","title":"XmlSchema.xml_one/3","type":"macro"},{"doc":"Declare an XML tag with a scalar value with type conversion.\nAny type accepted by Ecto.Schema can be used. Custom types\ncan be defined in the same manner. An xml_tag is terminal, for\ntags containing substructure use xml_one or xml_many.\nSee attributes section above for details on how to capture\ntag attributes.\n\n```\ndefmodule Example.Tag do\n  use XmlSchema, tag: \"a\"\n\n  xml do\n    xml_tag :k, :string\n  end\n\n  def doc do\n    \"\"\"\n       \n         one \n       \n    \"\"\"\n  end\n\n  def expect do\n    %Example.Tag{_attributes: nil, k: \"one\"}\n  end\n\n  def expect_xml do\n    \"\"\"\n     \n     \n       one \n     \n    \"\"\"\n    |> String.trim_trailing()\n  end\n\nend\n\n```","ref":"XmlSchema.html#xml_tag/3","title":"XmlSchema.xml_tag/3","type":"macro"},{"doc":"Type translates a string into an atom on xml parsing. Use\nwith caution, only for trusted documents. Can lead to\natom exhaustion.","ref":"XmlSchema.Type.Atom.html","title":"XmlSchema.Type.Atom","type":"module"},{"doc":"","ref":"XmlSchema.Type.Atom.html#cast/1","title":"XmlSchema.Type.Atom.cast/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Atom.html#dump/1","title":"XmlSchema.Type.Atom.dump/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Atom.html#embed_as/1","title":"XmlSchema.Type.Atom.embed_as/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Atom.html#equal?/2","title":"XmlSchema.Type.Atom.equal?/2","type":"function"},{"doc":"","ref":"XmlSchema.Type.Atom.html#load/1","title":"XmlSchema.Type.Atom.load/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Atom.html#type/0","title":"XmlSchema.Type.Atom.type/0","type":"function"},{"doc":"Converts float currency with cents into integer cents.","ref":"XmlSchema.Type.Cents.html","title":"XmlSchema.Type.Cents","type":"module"},{"doc":"","ref":"XmlSchema.Type.Cents.html#cast/1","title":"XmlSchema.Type.Cents.cast/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Cents.html#dump/1","title":"XmlSchema.Type.Cents.dump/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Cents.html#embed_as/1","title":"XmlSchema.Type.Cents.embed_as/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Cents.html#equal?/2","title":"XmlSchema.Type.Cents.equal?/2","type":"function"},{"doc":"","ref":"XmlSchema.Type.Cents.html#load/1","title":"XmlSchema.Type.Cents.load/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.Cents.html#type/0","title":"XmlSchema.Type.Cents.type/0","type":"function"},{"doc":"Trims input strings of whitespace. Can be a common occurance in\nsome Xml documents.","ref":"XmlSchema.Type.StringTrim.html","title":"XmlSchema.Type.StringTrim","type":"module"},{"doc":"","ref":"XmlSchema.Type.StringTrim.html#cast/1","title":"XmlSchema.Type.StringTrim.cast/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.StringTrim.html#dump/1","title":"XmlSchema.Type.StringTrim.dump/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.StringTrim.html#embed_as/1","title":"XmlSchema.Type.StringTrim.embed_as/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.StringTrim.html#equal?/2","title":"XmlSchema.Type.StringTrim.equal?/2","type":"function"},{"doc":"","ref":"XmlSchema.Type.StringTrim.html#load/1","title":"XmlSchema.Type.StringTrim.load/1","type":"function"},{"doc":"","ref":"XmlSchema.Type.StringTrim.html#type/0","title":"XmlSchema.Type.StringTrim.type/0","type":"function"},{"doc":"Raised when type conversion from xml string fails","ref":"XmlSchema.ParseError.html","title":"XmlSchema.ParseError","type":"exception"},{"doc":"# XmlSchema\n\n🧽 (MOP)\n\nXML Parser based on Ecto schemas\n\nParses XML documents directly into Ecto.Schema but with additional\nlayer of DSL to describe XML structure. All features from\nEcto.Schema can be used.\n\nPurpose is to ease access to CDATA and attributes on input at the\nsacrifice of precise access to all CDATA fragments that may exist\n(as occurs with indentation). Tag parsing and generation order is\npreserved based on the schema defined. Order of attributes are not\npreserved and are stored as a map excluding multiple instances of\nthe same attribute in a single tag instance.\n\nTerminal tags are available by default as bare values\n(strings, integers, etc.), but does not allow for attributes. A\ntuple container can be specified if attributes are needed resulting\nin values of `{tag-value, attributes-map}`. The syntax to create this\ncontainer is `{:param, type}`, where type is a base type (like `:string`,\nor a custom type).\n\n# Sample\n\n```elixir\ndefmodule Embed1 do\n  use XmlSchema\n\n  xml do\n    xml_tag :EventHorizon, :string\n    xml_one :Single, Single\n  end\nend\n```\n\n(Single is defined in tests/support/single.ex)\n\nAll XmlSchema instrumented modules provide `xml` outer declaration with inner\ndeclarations of:\n\n- `xml_tag` - A terminal XML node with CDATA values\n- `xml_one` - A tag that contains one substructure\n- `xml_many` - A tag that contains many of the named tag\n\nTags unspecified are ignored.\n\nAbove parses:\n\n```xml\n \n \n   No Light escapes \n   Bob \n   \n     Bob \n     123 Evergreen Terrace \n     Apt 123 \n   \n \n```\n\nand results in\n\n```\n%Embed1{\n    _attributes: %{\"creator\" => \"danj\"},\n    EventHorizon: \"No Light escapes\",\n    Single: %Single{\n      _attributes: %{\"complete\" => \"false\"},\n      Name: \"Bob\",\n      Address: [\"123 Evergreen Terrace\", \"Apt 123\"]\n    }\n  }\n```\n\nFor examples look at test/xml, test/support\n\nLicense\n\nCopyright (c) 2023 Triskelion, LLC\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in compliance with the License. You may obtain a copy of the License at https://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software distributed under the License is distributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html","title":"XmlSchema","type":"extras"}]}